import { CommandHandler } from '@nestjs/cqrs';
import { Messager } from '../../adapters/secondaries/messager';
import { UsersRepository } from '../../adapters/secondaries/users.repository';
import { DeleteUserCommand } from '../../commands/delete-user.command';
import { User } from '../entities/user';

@CommandHandler(DeleteUserCommand)
export class DeleteUserUseCase {
  constructor(
    private readonly repository: UsersRepository,
    private readonly messager: Messager,
  ) {}

  execute = async (command: DeleteUserCommand): Promise<User> => {
    try {
      const user = await this.repository.delete(command.uuid);
      this.messager.publish('user.delete', JSON.stringify({ uuid: user.uuid }));
      this.messager.publish(
        'logging.user.delete.info',
        JSON.stringify({ uuid: user.uuid }),
      );
      return user;
    } catch (error) {
      this.messager.publish(
        'logging.user.delete.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  };
}
